#include <SoftwareSerial.h>

// Incluimos librerias
#include <Adafruit_GFX.h> 
#include <Adafruit_NeoMatrix.h>
#include <Adafruit_NeoPixel.h>


SoftwareSerial miBT(10, 11);

// Definición de pines
#define PIN_MATRIZ 12
#define PIN_ARRIBA 3
#define PIN_DERECHA 4
#define PIN_ABAJO 5
#define PIN_IZQUIERDA 6
#define PIN_BUZZER 2

// Tamaño de la matriz LED y tamaños de objetos del juego (solo para la parte del movimiento)
#define ANCHO_MATRIZ 8
#define ALTO_MATRIZ 8
#define ANCHO_PADDLE 3  
#define ALTO_PADDLE 1
#define TAMANO_PELOTA 1

// Instancia de la matriz LED
Adafruit_NeoMatrix matriz = Adafruit_NeoMatrix(ANCHO_MATRIZ, ALTO_MATRIZ, PIN_MATRIZ,
  NEO_MATRIX_BOTTOM + NEO_MATRIX_LEFT +
  NEO_MATRIX_COLUMNS + NEO_MATRIX_PROGRESSIVE,
  NEO_GRB + NEO_KHZ800);

// Colores predefinidos
#define A matriz.Color(255, 255, 0)  // Amarillo
#define C matriz.Color(255, 0, 0)    // Rojo
#define V matriz.Color(0, 255, 0)    // Verde

// Matriz que define los niveles del juego
int NIVELES[8][8] = {
  {0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, V, V, 0, 0},
  {0, 0, A, V, V, V, V, 0},
  {0, A, A, 0, 0, 0, 0, 0},
  {0, A, A, 0, 0, 0, 0, 0},
  {0, 0, A, C, C, C, C, 0},
  {0, 0, 0, 0, C, C, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0},
};

// Variables del juego
int posicionPaddle = ANCHO_MATRIZ / 2 - ANCHO_PADDLE / 2;
int posicionPelotaX = ANCHO_MATRIZ / 2;
int posicionPelotaY = 1;
int velocidadPelota = 1.5; // Velocidad de la pelota (ajustable)
int velocidadPelotaX = 1;
int velocidadPelotaY = 1;
bool juegoEnCurso = false;
int puntaje = 0;
int tamanoPaddle = 3; // Tamaño inicial del paddle

int dificultad = 1;

int puntajeGanaste = 30; // Puntos que son necesarios para ganar

int barraSuperiorY = 0;
int tamanoBarra = ANCHO_MATRIZ;

unsigned long ultimoTiempoActualizacion = 0;

// Declaración de objetos y variables para la comunicación Bluetooth

char DATO = 0;

bool pixelesBarra[ANCHO_MATRIZ];

// Declaración de funciones
void verificarVictoria();
void imprimirMatriz(int IMAGEN[8][8]);


void setup() {
  matriz.begin();
  matriz.setTextWrap(false);
  matriz.setBrightness(50);
  matriz.setTextSize(1);

  pinMode(PIN_BUZZER, OUTPUT);
  
  // Inicializamos la comunicación serie Bluetooth
  miBT.begin(38400);
  
  mostrarIntroduccion();

  iniciarJuego();
}

// Función para mostrar la pantalla de introducción
void mostrarIntroduccion() {
  for (int color = 0; color < 256; color++) {
    for (int i = 0; i < matriz.width(); i++) {
      for (int j = 0; j < matriz.height(); j++) {
        matriz.drawPixel(i, j, matriz.ColorHSV(color * 65536 / 255, 255, 255));
      }
    }
    matriz.show();
    delay(10); // Ajusta la velocidad del arcoíris aquí
  }
  
  matriz.fillScreen(0); // Limpiar la pantalla después del arcoíris
  matriz.show();

  String texto = "RETRO TV -- BREAKOUT GAME";
  for (int cursorX = 0; cursorX <= (texto.length() + 6) * 4; cursorX = cursorX + 1) {
    matriz.setCursor(-cursorX + 6, 1);
    matriz.setTextColor(matriz.Color(255, 255, 255));
    matriz.print(texto);
    matriz.show();
    delay(100);
    matriz.clear();
  }
}


void iniciarJuego() {
  juegoEnCurso = true;
  puntaje = 0;
  posicionPelotaX = ANCHO_MATRIZ / 2;
  posicionPelotaY = 1;
  resetearVelocidadPelota();

  // Ajustamos la dificultad siempre a 1
  dificultad = 1;
  tamanoPaddle = 3;

  for (int i = 0; i < ANCHO_MATRIZ; i++) {
    pixelesBarra[i] = true;
  }
}

// Función para actualizar la posición de la pelota
void actualizarPelota() {
  posicionPelotaX += velocidadPelotaX;
  posicionPelotaY += velocidadPelotaY;

  if (posicionPelotaX <= 0 || posicionPelotaX >= ANCHO_MATRIZ - TAMANO_PELOTA) {
    velocidadPelotaX = -velocidadPelotaX;
    reproducirBuzzer();
  }

  if (posicionPelotaY <= 0) {
    velocidadPelotaY = -velocidadPelotaY;
    reproducirBuzzer();
  }

  if (posicionPelotaY >= ALTO_MATRIZ - TAMANO_PELOTA) {
    if (posicionPelotaX >= posicionPaddle && posicionPelotaX < posicionPaddle + tamanoPaddle) {
      velocidadPelotaY = -velocidadPelotaY;
      reproducirBuzzer();
      puntaje++;
      velocidadPelotaX += random(-1, 2);
    } else {
      juegoEnCurso = false;
      mostrarPantallaFinJuego();
    }
  }

  if (puntaje >= puntajeGanaste) {
    juegoEnCurso = false; // Detener el juego
    mostrarPantallaVictoria(); // Mostrar la pantalla de victoria
  }

  if (posicionPelotaY == barraSuperiorY && posicionPelotaX >= 0 && posicionPelotaX < tamanoBarra && pixelesBarra[posicionPelotaX]) {
    reproducirBuzzer();
    velocidadPelotaY = 1;
    puntaje++;
    pixelesBarra[posicionPelotaX] = false;
  }
}

// Función para resetear la velocidad de la pelota según la dificultad
void resetearVelocidadPelota() {
  if (dificultad == 1) {
    velocidadPelotaX = random(2) == 0 ? velocidadPelota : -velocidadPelota;
    velocidadPelotaY = velocidadPelota;
  } else if (dificultad == 2) {
    velocidadPelotaX = random(2) == 0 ? velocidadPelota : -velocidadPelota;
    velocidadPelotaY = velocidadPelota;
  } else if (dificultad == 3) {
    velocidadPelotaX = random(2) == 0 ? velocidadPelota : -velocidadPelota;
    velocidadPelotaY = velocidadPelota;
  }
}

// Función para dibujar el paddle en la matriz
void dibujarPaddle() {
  for (int i = 0; i < tamanoPaddle; i++) {
    matriz.drawPixel(posicionPaddle + i, ALTO_MATRIZ - 1, matriz.Color(0, 255, 0));
  }
}

// Función para dibujar la pelota en la matriz
void dibujarPelota() {
  matriz.drawPixel(posicionPelotaX, posicionPelotaY, matriz.Color(255, 0, 0));
}

// Función para dibujar la barra superior en la matriz
void dibujarBarra() {
  for (int i = 0; i < tamanoBarra; i++) {
    if (pixelesBarra[i]) {
      matriz.drawPixel(i, barraSuperiorY, matriz.Color(0, 0, 255));
    }
  }
}

// Función para reproducir el sonido del buzzer
void reproducirBuzzer() {
  tone(PIN_BUZZER, 1000, 100);
  delay(100);
  noTone(PIN_BUZZER);
}

// Función para mostrar la pantalla de fin de juego
void mostrarPantallaFinJuego() {
  tone(PIN_BUZZER, 1500, 500);

  String texto = ("PUNTOS:");
  for (int cursorX = 0; cursorX <= (texto.length() + 6) * 4; cursorX = cursorX + 1) {
    matriz.setCursor(-cursorX + 6, 1);
    matriz.setTextColor(matriz.Color(150, 150, 150));
    matriz.fillScreen(matriz.Color(150, 0, 0));
    matriz.print(texto);
    matriz.show();
    delay(100);
    matriz.clear();
  }

  String puntos = ("100");
  for (int cursorX = 0; cursorX <= (puntos.length() + 6) * 4; cursorX = cursorX + 1) {
    matriz.setCursor(-cursorX + 6, 1);
    matriz.setTextColor(matriz.Color(150, 150, 150));
    matriz.fillScreen(matriz.Color(150, 0, 0));
    matriz.print(puntaje);
    matriz.show();
    delay(100);
    matriz.clear();
  }

  delay(750);

   // Reinicia el juego
  iniciarJuego();
}

// Función para mostrar la pantalla de victoria
void mostrarPantallaVictoria() {
  tone(PIN_BUZZER, 1500, 500);

  String texto = ("GANASTE!!");
  for (int cursorX = 0; cursorX <= (texto.length() + 6) * 4; cursorX = cursorX + 1) {
    matriz.setCursor(-cursorX + 6, 1);
    matriz.setTextColor(matriz.Color(255, 255, 255));
    matriz.fillScreen(matriz.Color(0, 150, 0));
    matriz.print(texto);
    matriz.show();
    delay(100);
    matriz.clear();
  }
}

// Implementación de la función verificarVictoria
void verificarVictoria() {
  bool victoria = true;
  for (int i = 0; i < tamanoBarra; i++) {
    if (pixelesBarra[i]) {
      victoria = false;
      break;
    }
  }

  if (victoria) {
    juegoEnCurso = false;
    mostrarPantallaVictoria();
  }
}

// Función para imprimir una matriz en la matriz LED
void imprimirMatriz(int IMAGEN[8][8]) {
  for (int x = 0; x < 8; x++) {
    for (int y = 0; y < 8; y++) {
      matriz.drawPixel(x, y, IMAGEN[x][y]);
    }
  }
  matriz.show();
}


bool presentacionMostrada = false;

void loop() {
  while (!presentacionMostrada) {
    mostrarIntroduccion();
    if (miBT.available()) {
      char dato = miBT.read();
      if (dato == 'A') {
        presentacionMostrada = true; // Cambiar la bandera para salir del bucle de presentación
      }
    }
  }

  if (!juegoEnCurso) {
    while (miBT.available()) {
      char dato = miBT.read();
      if (dato == 'A') {
        presentacionMostrada = false; // Reiniciar la pantalla de presentación
        iniciarJuego();
        break;
      }
    }
    return; // Salir del loop si no estamos en juego
  }

  while (miBT.available()) {
    char dato = miBT.read();
    if (dato == 'L' && posicionPaddle > 0) {
      posicionPaddle--; // Mover el paddle a la izquierda
    } else if (dato == 'R' && posicionPaddle < ANCHO_MATRIZ - tamanoPaddle) {
      posicionPaddle++; // Mover el paddle a la derecha
    }
  }

  unsigned long tiempoActual = millis();
  if (tiempoActual - ultimoTiempoActualizacion >= 200 / dificultad) {
    actualizarPelota();
    ultimoTiempoActualizacion = tiempoActual;
  }

  matriz.clear();
  dibujarPaddle();
  dibujarPelota();
  dibujarBarra();
  matriz.show();
}

